import { Element, js2xml, xml2js } from "xml-js";

import { DigitalCalibrationCertificateType } from "./DCC";

export class DCCDocument {
  _declaration: unknown;
  digitalCalibrationCertificate: DigitalCalibrationCertificateType;

  constructor(el?: Partial<DCCDocument>) {
    this._declaration = el?._declaration ?? { _attr: { version: "1.0", encoding: "UTF-8" } };
    this.digitalCalibrationCertificate = new DigitalCalibrationCertificateType(el?.digitalCalibrationCertificate);
  }

  static fromXml(xml) {
    const parsed = <Partial<DCCDocument>>xml2js(xml, {
      compact: true,
      ignoreComment: true,
      trim: true,
      attributesKey: "_attr",
      elementNameFn: function (val) {
        return val.replace("dcc:", "").replace("si:", "");
      },
      textFn: function (val) {
        return val
          .split("\n")
          .map((x) => x.trim())
          .join("\n");
      },
    });

    return new DCCDocument(parsed);
  }

  toXML(): string {
    return js2xml(<Element>this, {
      compact: true,
      ignoreComment: true,
      spaces: 2,
      attributesKey: "_attr",
      elementNameFn: function (val, currentElementObj) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const currentElement = <any>currentElementObj;
        return `${currentElement?.namespace ? `${currentElement.namespace}:` : ""}${val}`;
      },
    });
  }
}
